/*
 * PDF text extractor based on PDFlib TET
 *
 * The text is written as UTF-16 or as UTF-32 in native byte order, depending
 * on the size of wchar_t.
 *
 * Due to the lack of portable conversions from narrow to wide character
 * the program assumes that the program arguments are encoded as UTF-8.
 *
 * $Id: image_extractor.cpp,v 1.2 2013/05/21 10:21:44 stm Exp $
 */

#include <iostream>
#include <iomanip>
#include <fstream>
#include <algorithm>

#include "tet.hpp"

using namespace std;
using namespace pdflib;

namespace
{

/*  global option list */
const wstring globaloptlist = L"searchpath={{../data}}";

/* document-specific  option list */
const wstring docoptlist = L"";

/* page-specific option list;
 * Since we extract only images we disable the text engine
 */
const wstring pageoptlist = L"skipengines={text}";

/* here you can insert basic image extract options (more below) */
const wstring baseimageoptlist = L"";

wstring get_wstring(const TET& tet, const string& utf8_string);

} // end of anonymous namespace

int main(int argc, char **argv)
{
    int pageno = 0;
    
    if (argc != 2)
    {
        wcerr << L"usage: image_extractor <infilename>" << endl;
        return(2);
    }

    try
    {
        TET tet;

        tet.set_option(globaloptlist);

        /*
         * Caution: For simplicity we assume that the program arguments are
         * encoded as UTF-8, which might not be true in all cases!
         */
        wstring outfilebase(get_wstring(tet, string(argv[1])));
        wstring doc_name(get_wstring(tet, string(argv[1])));
        const int doc = tet.open_document(doc_name, docoptlist);

        if (doc == -1)
        {
            wcerr << L"Error " << tet.get_errnum()
                << L" in " << tet.get_apiname() << L"(): "
                << tet.get_errmsg() << endl;
            return 2;
        }

        /* get number of pages in the document */
        const int n_pages = (int) tet.pcos_get_number(doc, L"length:pages");

        /* loop over pages in the document */
        for (pageno = 1; pageno <= n_pages; ++pageno)
        {
            const int page = tet.open_page(doc, pageno, pageoptlist);

            if (page == -1)
            {
                wcerr << L"Error " << tet.get_errnum()
                    << L" in " << tet.get_apiname()
                    << L"(): " << tet.get_errmsg() << endl;
                continue;                        /* try next page */
            }

            /* Retrieve all images on the page */
            const TET_image_info *ti;
            while ((ti = tet.get_image_info(page)) != 0)
            {
                wostringstream optlist;

                /* Report image geometry */
                wcout << L"page " << pageno << L": "
                        << (int) ti->width << L"x" << (int) ti->height
                        << L"pt, alpha=" << ti->alpha << L", beta="
                        << ti->beta << endl;

                /* Retrieve additional image properties with pCOS */
                optlist.str(L"");
                optlist << L"images[" << ti->imageid << L"]/Width";
                wcout << L"   id=" << ti->imageid << L", "
                        << (int) tet.pcos_get_number(doc, optlist.str())
                        << L"x";

                optlist.str(L"");
                optlist << L"images[" << ti->imageid << L"]/Height";
                wcout << (int) tet.pcos_get_number(doc,  optlist.str())
                        << L" pixel, ";

                optlist.str(L"");
                optlist << L"images[" << ti->imageid << L"]/colorspaceid";
                const int cs = (int) tet.pcos_get_number(doc, optlist.str());

                if (cs != -1)
                {
                    optlist.str(L"");
                    optlist << L"colorspaces[" << cs << L"]/components";
                    wcout << (int) tet.pcos_get_number(doc, optlist.str())
                        << L"x";

                    optlist.str(L"");
                    optlist << L"images[" << ti->imageid << L"]/bpc";
                    wcout << (int) tet.pcos_get_number(doc, optlist.str())
                        << L" bit ";

                    optlist.str(L"");
                    optlist << L"colorspaces[" << cs << L"]/name";
                    wcout << tet.pcos_get_string(doc, optlist.str()) << endl;
                }
                else
                {
                    /* cs==-1 may happen for some JPEG 2000 images. bpc,
                     * colorspace name and number of components are not
                     * available in this case.
                     */
                    wcout << L"JPEG2000" << endl;
                }

                /*
                 * Fetch image data and write it to a disk file. The
                 * output filename is generated from the input filename,
                 * page number and image ID.
                 */
                wostringstream imageoptlist;

                imageoptlist << baseimageoptlist << L" filename={"
                        << outfilebase << L"_p" << pageno << L"_I"
                        << ti->imageid << L"}";

                if (tet.write_image_file(doc, ti->imageid, imageoptlist.str())
                                                                        == -1)
                {
                    wcerr << L"Error " << tet.get_errnum()
                        << L" in " << tet.get_apiname()
                        << L"(): " << tet.get_errmsg() << endl;
                    continue;                  /* process next image */
                }
            }

            if (tet.get_errnum() != 0)
            {
                wcerr << L"Error " << tet.get_errnum()
                    << L" in " << tet.get_apiname()
                    << L"() on page " << pageno
                    << L": " << tet.get_errmsg() << endl;
            }

            tet.close_page(page);
        }

        tet.close_document(doc);
    }
    catch (TET::Exception &ex) {
        if (pageno == 0)
        {
            wcerr << L"Error " << ex.get_errnum()
                << L" in " << ex.get_apiname()
                << L"(): " << ex.get_errmsg() << endl;
        }
        else
        {
            wcerr << L"Error " << ex.get_errnum()
                << L" in " << ex.get_apiname()
                << L"() on page " << pageno
                << L": " << ex.get_errmsg() << endl;
        }
        return 2;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given UTF-8 string.
 */
wstring get_wstring(const TET& tet, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = tet.convert_to_unicode(L"auto", utf8_string,
                                                L"outputformat=utf16");
        break;

    case 4:
        wide_string = tet.convert_to_unicode(L"auto", utf8_string,
                                                L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *>(wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace
